package builtins_test

import (
	"context"
	"github.com/stretchr/testify/assert"
	"testing"
	"ucl.lmika.dev/ucl"
	"ucl.lmika.dev/ucl/builtins"
)

func TestLists_First(t *testing.T) {
	tests := []struct {
		desc    string
		eval    string
		want    any
		wantErr bool
	}{
		{desc: "firsts 1", eval: `lists:first [1 2 3 4 5] 0`, want: []any{}},
		{desc: "firsts 2", eval: `lists:first [1 2 3 4 5] 3`, want: []any{1, 2, 3}},
		{desc: "firsts 3", eval: `lists:first [1 2 3] 5`, want: []any{1, 2, 3}},
		{desc: "firsts 4", eval: `lists:first [1 2 3] 1`, want: []any{1}},
		{desc: "firsts 5", eval: `lists:first [1 2 3 4 5] -1`, want: []any{1, 2, 3, 4}},
		{desc: "firsts 6", eval: `lists:first [1 2 3 4 5] -3`, want: []any{1, 2}},
		{desc: "firsts 7", eval: `lists:first [1 2 3 4 5] -8`, want: []any{}},
		{desc: "firsts 8", eval: `lists:first (itrs:from [1 2 3 4 5]) 3`, want: []any{1, 2, 3}},
		{desc: "firsts 9", eval: `lists:first (itrs:from [1 2 3]) 5`, want: []any{1, 2, 3}},

		{desc: "err 1", eval: `lists:first (itrs:from [1 2 3]) -3`, wantErr: true},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			inst := ucl.New(
				ucl.WithModule(builtins.Itrs()),
				ucl.WithModule(builtins.Lists()),
			)
			res, err := inst.Eval(context.Background(), tt.eval)
			if tt.wantErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.want, res)
			}
		})
	}
}

func uclListOf(args ...any) *ucl.ListObject {
	newList := ucl.NewListObject()
	for _, arg := range args {
		switch t := arg.(type) {
		case int:
			newList.Append(ucl.IntObject(t))
		default:
			panic("unhandled type")
		}
	}
	return newList
}
